#!/usr/bin/python

from sys import argv, stderr
# stderr = open("/dev/null", "a")
import os
import re
import gdchart
import gd
# import Gnuplot
# import Numeric
from math import log10

class _Colors:
	def __rev(self, seq):
		if (len(seq) > 1):
			return (seq[-1],)+__rev(seq[:-1])
		else:
			return seq[:]
	def __invcol(self, cols):
		return tuple(map(lambda x: 0xffffff - x, cols))
	def __rotate(self, num, colors):
		col = colors * (num/len(colors))
		col = col + colors[:(num%len(colors))]
		return col
	def PastelColors(self, num):
		colors=()
		step=0xffffff/num
		for i in range(0,0xffffff,step):
			colors = colors + (i,)	
		return colors
	def Gradient(self, num,step,mask=0xffffff):
		colors = ()
		step = max(0xff/step,1)
		step = step + (step << 8) + (step << 16)
		for i in range(0x000000,0xffffff&mask,step&mask):
			colors = colors + (i,)
		# colors = + __rev(colors[1:-1])
		colors = self.__rotate(num, colors)
		return colors
	def Rainbow(self, num):
		colors = (0xff0000,0xffff00,0x00ff00,
		          0x00ffff,0x0000ff,0xff00ff)
		colors = self.__rotate(num, colors)
		return colors
	def SpamAndMail(self, spam, num):
		if spam > 0:
			spamcol = self.Gradient(spam,spam,0x00ff00)
			spamcol = tuple(map(lambda x: x + 0xff0000, spamcol))
			# spamcol = self.__rev(tuple(map(lambda x: 0xffffff - x, spamcol)))
			# spamcol = self.__rev(spamcol)
		else:
			spamcol = ()
		if num > spam:
			numcol = self.Gradient(num-spam+1,num-spam+1,0x00ffff)[1:]
			# numcol = tuple(map(lambda x: x / 2, self.__invcol(numcol)))
		else:
			numcol = ()
		return spamcol+numcol;

Colors = _Colors()

class LogfileData:
	SIZE_ARRAY = 0
	NUM_ARRAY = 1
	def __init__(self):
		self.reset()
	def reset(self):
		self.data = {}
		self.folders = {} # All folders
	def filter_name(self, name):
		if name[0] == '#':
			return None
		if name[0] == '*':
			return None
		return name
	def __add_folders(self, date):
		if len(self.folders):
			fsize = len(self.folders.values()[0][0])
			init_value = [0]*fsize
		else:
			init_value = []
		for folder in date.keys():
			if (not self.folders.has_key(folder)):
				print >>stderr, "New folder %s" % folder
				self.folders[folder] = (init_value[:],
							init_value[:])
		for folder, value in self.folders.items():
			size = value[LogfileData.SIZE_ARRAY]
			num = value[LogfileData.NUM_ARRAY]
			if (folder in date.keys()):
				vnum, vsize = date[folder]
			else:
				vnum, vsize = 0, 0
			size.append(vsize)
			num.append(vnum)
	def __sum_up(self, data):
		res = []
		latest = data[0]
		res.append(latest)
		for array in data[1:]:
			array = map(lambda x,y: x+y, latest, array)
			latest = array
			res.append(array)
		return res
	def add_file(self, key, name):
		d = {}
		f = open(name)
		for line in f.xreadlines():
			line = line.strip()
			res = re.split('\s*', line, 2)
			folder = res[2].strip()
			value = (int(res[1]),
				float(res[0])/(1024*1024))
			folder = self.filter_name(folder)
			if not folder: continue
			if d.has_key(folder):
				d[folder] = (d[folder][0] + value[0],
					d[folder][1] + value[1])
			else:
				d[folder] = value
		self.data[key] = d
	def process_data(self):
		self.keys = self.data.keys()
		self.keys.sort()
		for key in self.keys:
			self.__add_folders(self.data[key])
		key = -1
		print >>stderr, "Last data:", self.keys[key]
		for folder, value in self.folders.items():
			size = value[LogfileData.SIZE_ARRAY][key]
			num = value[LogfileData.NUM_ARRAY][key]
			print >>stderr, folder, ":", size,"/",num	
	def get_spams(self):
		folders = self.folders.keys()
		folders.sort()
		return filter(lambda x: x[:4] == 'SPAM' or x == 'UNSURE',
		       folders)
	def __sorted_array(self, array_type):
		folders = self.folders.items()
		folders.sort(lambda x, y: cmp(x[0], y[0]))
		return map(lambda x: x[1][array_type], folders)
	def get_size(self):
		return self.__sorted_array(LogfileData.SIZE_ARRAY)
	def get_numbers(self):
		return self.__sorted_array(LogfileData.NUM_ARRAY)
	def get_labels(self):
		return self.keys
	def get_folders(self):
		folders = self.folders.keys()
		folders.sort()
		return folders

class Proclog:
	def __init__(self):
		self.deffont = gd.gdFontSmall
		self.columns = 3
		self.target = os.path.join(os.environ['HOME'],'public_html')
	def __c2t(self, color):
		red = color >> 16
		green = (color >> 8) & 0xff
		blue = color & 0xff
		return red, green, blue
	def __alloc_color(self, img, color):
		col = self.__c2t(color)
		idx = img.colorExact(col)
		if (idx != -1): return idx
		idx = img.colorAllocate(col)
		if (idx != -1): return idx
		print "Warning: can't allocate color!"
		idx = img.colorClosest(col)
		if (idx != -1): return idx
		raise "Can't find a slot for color %06x!" % color
	def __get_colors(self):
		nspams = len(self.data.get_spams())
		nfolders = len(self.data.get_folders())
		return Colors.SpamAndMail(nspams, nfolders)
	def __draw_charts(self):
		chart = gdchart.Area();
		chart.title='debit de courriel'
		chart.bg_color="white"
		chart.width = 500
		chart.height = 500
		chart.setLabels(self.data.get_labels())
		chart.set_color = self.__get_colors()

		chart.ytitle='Mo/semaine'
		stderr.flush()
		data = self.data.get_size()
		nfolders = len(self.data.get_folders())
		nlabels = len(self.data.get_labels())
		if nfolders !=  len(data):
			raise ("Wrong Data Size: %d" % len(data))
		else:
			print >>stderr, "OK"
		for fitem in data:
		       if len(fitem) != nlabels:
			       raise "Not enough data!: %d" % len(fitem)
		chart.setData(*data)
		chart.draw(os.path.join(self.target,'size.png'))

		chart.ytitle='courriel/semaine'
		data = self.data.get_numbers()
		chart.setData(*data)
		chart.draw(os.path.join(self.target,'num.png'))

	def __draw_legend(self):
		folders = self.data.get_folders()
		lines = len(folders)/self.columns
		if (len(folders)%self.columns != 0):
			lines = lines+1
		dx, dy = gd.fontstrsize(self.deffont, 'test')
		sx, sy = (600, (lines+1)*(dy+5))
		img = gd.image((sx,sy))
		white = self.__alloc_color(img, 0xffffff)
		black = self.__alloc_color(img, 0)
		img.fill((0,0), white)
		colors = self.__get_colors()
		for idx in xrange(len(folders)):
			folder = folders[idx]
			color = self.__alloc_color(img, colors[idx])
			(dx, dy) = gd.fontstrsize(self.deffont, folder)
			y = 5+(dy+5)*(idx % lines)
			x = 5+(sx/self.columns+5)*(idx/lines)
			img.rectangle((x,y), (x+dy, y+dy), black, color)
			img.string(self.deffont, (x+dy+5, y), folder, black)
		img.writePng(os.path.join(self.target,'legend.png'))
	def draw(self, data):
		self.data = data
		self.data.process_data()
		self.__draw_charts()
		self.__draw_legend()

class MyLogfileData(LogfileData):
	def __init__(self):
		LogfileData.__init__(self)
	def filter_name(self, name):
		if name[:8] == 'formail ':
			return None
		if name == 'sa/':
			return 'SPAM:spam/' #+ ':assassin/'
		if name == 'spam-bogofilter/':
			return 'SPAM' + ':bogofilter/'
		if name == 'virus/':
			return 'SPAM:spam/' #+ ':virus/'
		if name == 'spam/unsure/':
			return 'UNSURE'
		if name == 'spam/virus/':
			return 'SPAM:spam/'
		if name == 'spam/assassin/':
			return 'SPAM:spam/'
		if name[:5] == 'spam/':
			return 'SPAM' + ':' + name[5:]
		if name == '/var/spool/mail/fabien' or \
		       name == '/var/mail/fabien' or \
		       name == '/home/fabien/Mail/inbox/' or \
		       name == '/home/fabien/Mail/mbox/':
			return '_inbox'
		if name == '/dev/null' or \
		       name[:8] == 'sentmail' or \
		       name[:18] == '/usr/sbin/sendmail':
			return None
		return LogfileData.filter_name(self, name)

if (__name__ == '__main__'):
	dict = {}
	files = argv[1:]
	data = MyLogfileData();
	for file in files[-104:]:
		key = file[-14:-4]
		data.add_file(key, file)
	Proclog().draw(data)
